@@ -22,7 +22,9 @@
 
 #include <config.h>
 #include <sys/types.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
 #include <stdio.h>
 #include <string.h>
 
@@ -1149,6 +1151,11 @@ async_recv_cb (ORBitAsyncQueueEntry *aqe)
 		CORBA_exception_set_system (ev, ex_CORBA_COMM_FAILURE,
 					    aqe->completion_status);
 
+	if (aqe->mqe.cnx &&
+	    aqe->mqe.cnx->parent.status == LINK_TIMEOUT)
+		CORBA_exception_set_system (ev, ex_CORBA_TIMEOUT,
+					    aqe->completion_status);
+
 	if (aqe->fn)
 		aqe->fn (aqe->obj, aqe->m_data, aqe, aqe->user_data, ev);
 
@@ -1393,7 +1400,7 @@ void
 ORBit_small_connection_unref (ORBitConnection *cnx)
 {
 	if (cnx)
-		giop_connection_unref (cnx);
+		giop_connection_unref (GIOP_CONNECTION (cnx));
 }
 
 void
